#include "alsa.h"
#include <stdio.h>


snd_pcm_uframes_t AlsaInit(snd_pcm_t ** soundDevice, const char *name) {
  int i;
  int err,resample;
  unsigned int actualRate = 44100;
  snd_pcm_hw_params_t *hw_params;
  snd_pcm_uframes_t bufferSize;

  if (name==NULL) {
      // Try to open the default device
      err=snd_pcm_open(soundDevice, "plughw:0,0", SND_PCM_STREAM_PLAYBACK, 0 );
  } else {
      // Open the device we were told to open.
      err=snd_pcm_open(soundDevice, name, SND_PCM_STREAM_PLAYBACK, 0);
  }
  if (err<0) printf("Err: pcm_open\n");

  // Allocate the hardware parameter structure.
  err=snd_pcm_hw_params_malloc(&hw_params);
  if (err<0) printf("Err: hw_params_malloc\n");
  err=snd_pcm_hw_params_any(*soundDevice, hw_params);
  if (err<0) printf("Err: hw_params_any\n");
  err=snd_pcm_hw_params_set_rate_resample(*soundDevice, hw_params, resample);
  if (err<0) printf("Err: set_resample_rate\n");
  err=snd_pcm_hw_params_set_access(*soundDevice, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
  if (err<0) printf("Err: set_access\n");
  err=snd_pcm_hw_params_set_format(*soundDevice, hw_params, SND_PCM_FORMAT_S16_LE);
  if (err<0) printf("Err: set_format\n");
  err=snd_pcm_hw_params_set_channels(*soundDevice, hw_params, 1);
  if (err<0) printf("Err: set_channels\n");
  err=snd_pcm_hw_params_set_rate_near(*soundDevice, hw_params, &actualRate, 0);
  if (err<0) printf("Err: set_rate_near\n");
  err=snd_pcm_hw_params(*soundDevice, hw_params);
  if (err<0) printf("Err: phw_params\n");
  err=snd_pcm_hw_params_get_buffer_size(hw_params, &bufferSize);
  if (err<0) printf("Err: get_buffer_size\n");
  snd_pcm_hw_params_free(hw_params);

  err=snd_pcm_prepare(*soundDevice);
  if (err<0) printf("Err: pcm_prepare\n");

  return bufferSize;
}

void AlsaUninit(snd_pcm_t *soundDevice) {
  snd_pcm_close(soundDevice);
}


