#include <linux/input.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "alsa.h"
#include <math.h>

#define ACCEL_EVENT_FILE "/dev/input/event2"

#define BUFF_SIZE 1024
#define max(a,b) (a>b?a:b)
#define min(a,b) (a<b?a:b)

int main(int argc, char **argv) {
    int accelfd;
    snd_pcm_uframes_t buffsize;
    snd_pcm_t *soundDevice;
    short buff[BUFF_SIZE];
    struct input_event ev;
    int i;
    int x,y,z;
    long s;
    float intens;
    
    accelfd=open(ACCEL_EVENT_FILE,O_RDONLY|O_NONBLOCK);
    if (accelfd<0) {
	printf("Couldn't open %s.\n",ACCEL_EVENT_FILE);
	exit(1);
    }
    
    buffsize=AlsaInit(&soundDevice,NULL);
    if (buffsize<=0) {
	printf("Alsa init failed.\n");
	exit(1);
    }
    
    
    while(1) {
	while(read(accelfd,&ev,sizeof(ev))==sizeof(ev)) {
	    if (ev.type==EV_SYN && ev.code==SYN_REPORT) {
		//Event values are all sent. Calc lightsaber intensity.
		intens=sqrt(x*x+y*y+z*z)/4000;
//		printf("X: %i Y: %i Z: %i --> %f\n",x,y,z,intens);
	    }
	    if (ev.type==EV_REL && ev.code==REL_X) x=ev.value;
	    if (ev.type==EV_REL && ev.code==REL_Y) y=ev.value;
	    if (ev.type==EV_REL && ev.code==REL_Z) z=ev.value;
	}

	for (i=0; i<BUFF_SIZE; i++) {
	    s=sin(3.141592f*2*i/128)*(32767.0);
	    s=(float)s*intens*intens;
	    
	    s+=(i>BUFF_SIZE/2?32767:-32767)*max(intens-0.4,0)*.8;
	    
	    //clip
	    if (s>32767) s=32767;
	    if (s<-32767) s=-32767;
	    
	    buff[i]=s;
//	    printf("%i, %i\n",i,buff[i]);
	}
	while (snd_pcm_delay(soundDevice,&i)>=0 && i>BUFF_SIZE*2) ;
	snd_pcm_writei(soundDevice, buff, BUFF_SIZE);
	snd_pcm_wait(soundDevice,1000);
    }

    AlsaUninit(soundDevice);
}